Type
  Date = Word;
  Time = LongInt;

  DateTimeRec = record
   D: Date;
   T: Time;
  end;

  DateTime = record
    Year, Month, Day, Hour, Min, Sec: Word;
  end;

  DayType = (Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday);

  TDateTimeRec = record
    FTime,FDate: SmallWord;
  end;

const
  MinYear  = 1900;
  MaxYear  = 2078;
  MinDate  = $0000;        {= 01/01/1900}
  MaxDate  = $FF62;        {= 12/31/2078}
  Date1900 = $0000;        {= 01/01/1900}
  Date1980 = $7223;        {= 01/01/1980}
  Date2000 = $8EAC;        {= 01/01/2000}
  BadDate  = $FFFF;

  MinTime = 0;             {= 00:00:00 am}
  MaxTime = 86399;         {= 23:59:59 pm}
  BadTime = $FFFFFFFF;
  First2Months = 58;       {1900 was not a leap year}
  FirstDayOfWeek = Monday; {01/01/1900 was a Monday}

  SecondsInDay = 86400;
  SecondsInHour = 3600;    {number of seconds in an hour}
  SecondsInMinute = 60;    {number of seconds in a minute}
  HoursInDay = 24;         {number of hours in a day}
  MinutesInHour = 60;      {number of minutes in an hour}

  GmtHourOffset : Integer = 0;   {Default no GMT adjustments}
  Threshold2000 : Integer = 1900;



procedure IncDateTime(
  var DT1, DT2 : DateTimeRec; Days : Integer; Secs : LongInt);
  {-Increment (or decrement) DT1 by the specified number of days and seconds
    and put the result in DT2}
begin
  DT2 := DT1;

  {date first}
  Inc(Integer(DT2.D), Days);

  if Secs < 0 then begin
    {change the sign}
    Secs := -Secs;

    {adjust the date}
    Dec(DT2.D, Secs div SecondsInDay);
    Secs := Secs mod SecondsInDay;

    if Secs > DT2.T then begin
      {subtract a day from DT2.D and add a day's worth of seconds to DT2.T}
      Dec(DT2.D);
      Inc(DT2.T, SecondsInDay);
    end;

    {now subtract the seconds}
    Dec(DT2.T, Secs);
  end
  else begin
    {increment the seconds}
    Inc(DT2.T, Secs);

    {adjust date if necessary}
    Inc(DT2.D, DT2.T div SecondsInDay);

    {force time to 0..SecondsInDay-1 range}
    DT2.T := DT2.T mod SecondsInDay;
  end;
end;

procedure DateToDMY(Julian : Date; var Day, Month, Year : Integer);
  {-Convert from a julian date to month, day, year}
var
  I : LongInt;
begin
  if Julian = BadDate then begin
    Day := 0;
    Month := 0;
    Year := 0;
  end
  else if Julian <= First2Months then begin
    Year := MinYear;
    if Julian <= 30 then begin
      Month := 1;
      Day := Succ(Julian);
    end
    else begin
      Month := 2;
      Day := Julian-30;
    end;
  end
  else begin
    I := (4*LongInt(Julian-First2Months))-1;
    Year := I div 1461;
    I := (5*((I mod 1461) div 4)) + 2;
    Month := I div 153;
    Day := ((I mod 153)+5) div 5;
    if Month < 10 then
      Inc(Month, 3)
    else begin
      Dec(Month, 9);
      Inc(Year);
    end;
    Inc(Year, MinYear);
  end;
end;

procedure TimeToHMS(T : Time; var Hours, Minutes, Seconds : Byte);
  {-Convert a Time variable to Hours, Minutes, Seconds}
begin
  if T = BadTime then begin
    Hours := 0;
    Minutes := 0;
    Seconds := 0;
  end
  else begin
    Hours := T div SecondsInHour;
    Dec(T, LongInt(Hours)*SecondsInHour);
    Minutes := T div SecondsInMinute;
    Dec(T, LongInt(Minutes)*SecondsInMinute);
    Seconds := T;
  end;
end;

procedure PackTime(var T: DateTime; var P: Longint);
var
  FDateTime: TDateTimeRec absolute P;
begin
  with T,FDateTime do
  begin
    FDate := (Year - 1980) shl 9 + Month shl 5 + Day;
    FTime := Hour shl 11 + Min shl 5 + (Sec div 2);
  end;
end;

function DMYtoDate(Day, Month, Year: Integer) : Date;
  {-Convert from day, month, year to a julian date}
begin
  if Word(Year) < 100 then begin
    Inc(Year, 1900);
    if Year < Threshold2000 then
      Inc(Year, 100);
  end;

  if (Year = MinYear) and (Month < 3) then
    if Month = 1 then
      DMYtoDate := Pred(Day)
    else
      DMYtoDate := Day+30
  else begin
    if Month > 2 then
      Dec(Month, 3)
    else begin
      Inc(Month, 9);
      Dec(Year);
    end;
    Dec(Year, MinYear);
    DMYtoDate :=
        ((LongInt(Year)*1461) div 4)+
        (((153*Month)+2) div 5)+Day+First2Months;
  end;
end;


function UnixDateToPack(YMTime: LongInt): LongInt;
  {-Return a file time stamp in packed format from a Ymodem time stamp}
var
  DT: DateTime;
  DTR: DateTimeRec;
  Ptime: LongInt;
  H,M,S: Byte;
  StartDate: DateTimeRec;  {Set to 1/1/1970 00:00 GMT}
begin
   with StartDate do begin
     T := 0;
     D := DMYToDate(1, 1, 1970);
   end;

  {Add the time stamp to StartDate}
  IncDateTime(StartDate, DTR, 0, YMTime);

  {Add the GMT hour offset}
  IncDateTime(DTR, DTR, 0, 3600 * GmtHourOffset);

  {Convert to DT format}
  with DT do begin
    DateToDMY(DTR.D, Integer(Day), Integer(Month), Integer(Year));
    TimeToHMS(DTR.T, H, M, S);
    Hour:=H;
    Min:=M;
    Sec:=S;
  end;

  {Convert to packed format}
  PackTime(DT, Ptime);
  UnixDateToPack:=Ptime;
end;

Type
 TMsDateTime = record
  Mode : byte;
  case byte of
   0 : (Long : Comp;);
   1 : (Low, High  : Longint;);
 end;

Const
 UnixStartInMS : TMSDateTime = (Mode : 0; Low:-717324288; High:27111902);

function MSDateTimeToPack(MSTime: Comp): LongInt;
var
 tmpMS:TMSDateTime;
begin
 tmpMS.Long:=MSTime-UnixStartInMS.Long;
 tmpMS.Long:=tmpMS.Long / 10000000;
 Result:=UnixDateToPack(tmpMS.Low);
end;
